/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.orchestra.lib.jsf;

import java.util.Map;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.apache.myfaces.orchestra.CoreConfig;
import org.apache.myfaces.orchestra.conversation.ConversationContext;
import org.apache.myfaces.orchestra.conversation.ConversationManager;

/**
 * RequestHandler that ensures that only one thread is processing
 * each ConversationContext at a time.
 * 
 * @since 1.1
 */
class ContextLockRequestHandler implements RequestHandler
{
    ConversationContext context;

    public void init(FacesContext facesContext) throws FacesException
    {
        if (getSerializeRequests(facesContext))
        {
            // Note that ConversationManager.getInstance requires the FrameworkAdapter
            // to be initialised...
            //
            // ?? Why is false passed here ??
            ConversationManager manager = ConversationManager.getInstance(false);
            if (manager != null)
            {
                context = manager.getCurrentConversationContext();
                if (context != null)
                {
                    try
                    {
                        context.lockInterruptablyForCurrentThread();
                    }
                    catch(InterruptedException e)
                    {
                        throw new FacesException(e);
                    }
                }
            }
        }
    }

    public void deinit() throws FacesException
    {
        if (context != null)
        {
            context.unlockForCurrentThread();
        }
    }
    
    private boolean getSerializeRequests(FacesContext facesContext)
    {
        ExternalContext ec = facesContext.getExternalContext();

        // Check for deprecated setting via the OrchestraServletFilter.
        Map reqScope = ec.getRequestMap();
        Boolean serializeRequests = (Boolean) reqScope.get(CoreConfig.SERIALIZE_REQUESTS);
        if (serializeRequests != null)
        {
            return serializeRequests.booleanValue();
        }

        // Check for the normal global init param; true unless "false" is specified 
        String value = ec.getInitParameter(CoreConfig.SERIALIZE_REQUESTS);
        return !"false".equals(value); // NON-NLS
    }
}

